#region Using declarations
using System;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
using TradingStudies.NinjaScript.Utility;
#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    /// <summary>
    /// TSSuperTrend Indicator
    /// Version 2.0
    /// Visual options for bar colors added by Elliott Wave 12/08.
    /// </summary>
    [Description("TSSuperTrend Indicator developed by Roonius")]
    public class TSSuperTrend : Indicator
    {
        #region Variables
        // Wizard generated variables
        private int length = 14; // Default setting for Length
        private double multiplier = 2.618; // Default setting for Multiplier
        private bool showArrows = false;
        private bool colorBars = false;
        private BoolSeries trend;
		private double avg = 0;
		private double offset = 0;
		private MovingAverageType maType = MovingAverageType.HMA;
		private SuperTrendMode smode = SuperTrendMode.ATR;
		private int smooth = 14;
        private Color barColorUp = Color.Blue;
        private Color barColorDown = Color.Red;
        private Color tempColor, prevColor;
        #endregion

        /// <summary>
        /// This method is used to configure the indicator and is called once before any bar data is loaded.
        /// </summary>
        protected override void Initialize()
        {
			Input = Median;
            Add(new Plot(Color.Green, PlotStyle.Hash, "UpTrend"));
            Add(new Plot(Color.Red, PlotStyle.Hash, "DownTrend"));
            CalculateOnBarClose = true;
            Overlay = true;
            PriceTypeSupported = true;
            trend = new BoolSeries(this);
        }

        /// <summary>
        /// Called on each bar update event (incoming tick)
        /// </summary>
        protected override void OnBarUpdate()
        {
            // Use this method for calculating your indicator values. Assign a value to each
            // plot below by replacing 'Close[0]' with your own formula.
            if (CurrentBar < 1)
            {
                trend.Set(true);
                UpTrend.Set(Close[0]);
                DownTrend.Set(Close[0]);
                return;
            };
			switch (maType) 
			{
				case MovingAverageType.SMA:
					avg = SMA(Input, smooth)[0];
					break;
				case MovingAverageType.SMMA:
					avg = SMMA(Input, smooth)[0];
					break;
				case MovingAverageType.TMA:
					avg = TMA(Input, smooth)[0];
					break;
				case MovingAverageType.WMA:
					avg = WMA(Input, smooth)[0];
					break;
				case MovingAverageType.VWMA:
					avg = VWMA(Input, smooth)[0];
					break;
				case MovingAverageType.TEMA:
					avg = TEMA(Input, smooth)[0];
					break;
				case MovingAverageType.HMA:
					avg = HMA(Input, smooth)[0];
					break;
				case MovingAverageType.VMA:
					avg = VMA(Input, smooth, smooth)[0];
					break;
				default:
					avg = EMA(Input, smooth)[0];
					break;
			}
			switch (smode)
			{
				case SuperTrendMode.ATR:
					offset = ATR(length)[0] * Multiplier;
					break;
				case SuperTrendMode.Adaptive:
					offset = ATR(length)[0] * HomodyneDiscriminator(Median)[0] / 10;
					break;
				default:
					offset = DTT(length, Multiplier);
					break;
			}
            if (FirstTickOfBar)
                prevColor = tempColor;
            if (Close[0] > DownTrend[1])
                trend.Set(true);
            else
                if (Close[0] < UpTrend[1])
                    trend.Set(false);
                else
                    trend.Set(trend[1]);
            if (trend[0] && !trend[1])
            {
                UpTrend.Set(avg - offset);
                if (Plots[0].PlotStyle == PlotStyle.Line) UpTrend.Set(1, DownTrend[1]);
                tempColor = barColorUp;
                if (ShowArrows) DrawArrowUp(CurrentBar.ToString(), true, 0, UpTrend[0] - TickSize, barColorUp);
            }
            else
                if (!trend[0] && trend[1])
                {
                    DownTrend.Set(avg + offset);
                    if (Plots[1].PlotStyle == PlotStyle.Line) DownTrend.Set(1, UpTrend[1]);
                    tempColor = barColorDown;
                    if (ShowArrows) DrawArrowDown(CurrentBar.ToString(), true, 0, DownTrend[0] + TickSize, barColorDown);

                }
                else
                {
                    if (trend[0])
                        UpTrend.Set((avg - offset) > UpTrend[1] ? (avg - offset) : UpTrend[1]);
                    else
                        DownTrend.Set((avg + offset) < DownTrend[1] ? (avg + offset) : DownTrend[1]);
                    RemoveDrawObject(CurrentBar.ToString());
                    tempColor = prevColor;
                }

            if (colorBars)
            {
                CandleOutlineColor = tempColor;
                if (Open[0] < Close[0] && ChartControl.ChartStyleType == ChartStyleType.CandleStick)
                {
                    BarColor = Color.Transparent;
                }
                else
                {
                    BarColor = tempColor;
                }
            }
        }
		private double DTT(int nDay, double mult)
		{
			double HH = MAX(High, nDay)[0];
			double HC = MAX(Close,nDay)[0];
			double LL = MIN(Low, nDay)[0];
			double LC = MIN(Close, nDay)[0];
			
			return mult * Math.Max((HH - LC),(HC - LL));
		}

        #region Properties
        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries UpTrend
        {
            get { return Values[0]; }
        }

        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries DownTrend
        {
            get { return Values[1]; }
        }

        [Description("ATR/DT Period")]
        [Category("Parameters")]
        [Gui.Design.DisplayName("02. Period")]
        public int Length
        {
            get { return length; }
            set { length = Math.Max(1, value); }
        }
		
        [Description("Moving Average Type for smoothing")]
        [Category("Parameters")]
        [Gui.Design.DisplayName("04. Moving Average Type")]
        public MovingAverageType MAType
        {
            get { return maType; }
            set { maType = value; }
        }
		
        [Description("SuperTrendMode")]
        [Category("Parameters")]
        [Gui.Design.DisplayName("01. SuperTrend Mode")]
        public SuperTrendMode STMode
        {
            get { return smode; }
            set { smode = value; }
        }
		
        [Description("Smoothing Period")]
        [Category("Parameters")]
        [Gui.Design.DisplayName("05. SmoothingPeriod (MA)")]
        public int Smooth
        {
            get { return smooth; }
            set { smooth = Math.Max(1, value); }
        }
        [Description("Show Arrows when Trendline is violated?")]
        [Category("Visual")]
        [Gui.Design.DisplayName("02. Show Arrows?")]
        public bool ShowArrows
        {
            get { return showArrows; }
            set { showArrows = value; }
        }

        [Description("ATR Multiplier")]
        [Category("Parameters")]
        [Gui.Design.DisplayName("03. Multiplier")]
        public double Multiplier
        {
            get { return multiplier; }
            set { multiplier = Math.Max(0.0001, value); }
        }

        [Description("Color the bars in the direction of the trend?")]
        [Category("Visual")]
        [Gui.Design.DisplayName("03. Color Bars?")]
        public bool ColorBars
        {
            get { return colorBars; }
            set { colorBars = value; }
        }


        /// <summary>
        /// </summary>
        [XmlIgnore()]
        [Description("Color of up bars.")]
        [Category("Visual")]
        [Gui.Design.DisplayNameAttribute("04. Up color")]
        public Color BarColorUp
        {
            get { return barColorUp; }
            set { barColorUp = value; }
        }

        /// <summary>
        /// </summary>
        [Browsable(false)]
        public string barColorUpSerialize
        {
            get { return NinjaTrader.Gui.Design.SerializableColor.ToString(barColorUp); }
            set { barColorUp = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
        }

        /// <summary>
        /// </summary>
        [XmlIgnore()]
        [Description("Color of down bars.")]
        [Category("Visual")]
        [Gui.Design.DisplayNameAttribute("05. Down color")]
        public Color BarColorDown
        {
            get { return barColorDown; }
            set { barColorDown = value; }
        }

        /// <summary>
        /// </summary>
        [Browsable(false)]
        public string barColorDownSerialize
        {
            get { return NinjaTrader.Gui.Design.SerializableColor.ToString(barColorDown); }
            set { barColorDown = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
        }
        [Browsable(false)]
        public BoolSeries Trend
        {
            get { return trend; }
        }

        #endregion
    }
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private TSSuperTrend[] cacheTSSuperTrend = null;

        private static TSSuperTrend checkTSSuperTrend = new TSSuperTrend();

        /// <summary>
        /// TSSuperTrend Indicator developed by Roonius
        /// </summary>
        /// <returns></returns>
        public TSSuperTrend TSSuperTrend(int length, MovingAverageType mAType, double multiplier, int smooth, SuperTrendMode sTMode)
        {
            return TSSuperTrend(Input, length, mAType, multiplier, smooth, sTMode);
        }

        /// <summary>
        /// TSSuperTrend Indicator developed by Roonius
        /// </summary>
        /// <returns></returns>
        public TSSuperTrend TSSuperTrend(Data.IDataSeries input, int length, MovingAverageType mAType, double multiplier, int smooth, SuperTrendMode sTMode)
        {
            checkTSSuperTrend.Length = length;
            length = checkTSSuperTrend.Length;
            checkTSSuperTrend.MAType = mAType;
            mAType = checkTSSuperTrend.MAType;
            checkTSSuperTrend.Multiplier = multiplier;
            multiplier = checkTSSuperTrend.Multiplier;
            checkTSSuperTrend.Smooth = smooth;
            smooth = checkTSSuperTrend.Smooth;
            checkTSSuperTrend.STMode = sTMode;
            sTMode = checkTSSuperTrend.STMode;

            if (cacheTSSuperTrend != null)
                for (int idx = 0; idx < cacheTSSuperTrend.Length; idx++)
                    if (cacheTSSuperTrend[idx].Length == length && cacheTSSuperTrend[idx].MAType == mAType && Math.Abs(cacheTSSuperTrend[idx].Multiplier - multiplier) <= double.Epsilon && cacheTSSuperTrend[idx].Smooth == smooth && cacheTSSuperTrend[idx].STMode == sTMode && cacheTSSuperTrend[idx].EqualsInput(input))
                        return cacheTSSuperTrend[idx];

            TSSuperTrend indicator = new TSSuperTrend();
            indicator.BarsRequired = BarsRequired;
            indicator.CalculateOnBarClose = CalculateOnBarClose;
            indicator.Input = input;
            indicator.Length = length;
            indicator.MAType = mAType;
            indicator.Multiplier = multiplier;
            indicator.Smooth = smooth;
            indicator.STMode = sTMode;
            indicator.SetUp();

            TSSuperTrend[] tmp = new TSSuperTrend[cacheTSSuperTrend == null ? 1 : cacheTSSuperTrend.Length + 1];
            if (cacheTSSuperTrend != null)
                cacheTSSuperTrend.CopyTo(tmp, 0);
            tmp[tmp.Length - 1] = indicator;
            cacheTSSuperTrend = tmp;
            Indicators.Add(indicator);

            return indicator;
        }

    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// TSSuperTrend Indicator developed by Roonius
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.TSSuperTrend TSSuperTrend(int length, MovingAverageType mAType, double multiplier, int smooth, SuperTrendMode sTMode)
        {
            return _indicator.TSSuperTrend(Input, length, mAType, multiplier, smooth, sTMode);
        }

        /// <summary>
        /// TSSuperTrend Indicator developed by Roonius
        /// </summary>
        /// <returns></returns>
        public Indicator.TSSuperTrend TSSuperTrend(Data.IDataSeries input, int length, MovingAverageType mAType, double multiplier, int smooth, SuperTrendMode sTMode)
        {
            return _indicator.TSSuperTrend(input, length, mAType, multiplier, smooth, sTMode);
        }

    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// TSSuperTrend Indicator developed by Roonius
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.TSSuperTrend TSSuperTrend(int length, MovingAverageType mAType, double multiplier, int smooth, SuperTrendMode sTMode)
        {
            return _indicator.TSSuperTrend(Input, length, mAType, multiplier, smooth, sTMode);
        }

        /// <summary>
        /// TSSuperTrend Indicator developed by Roonius
        /// </summary>
        /// <returns></returns>
        public Indicator.TSSuperTrend TSSuperTrend(Data.IDataSeries input, int length, MovingAverageType mAType, double multiplier, int smooth, SuperTrendMode sTMode)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.TSSuperTrend(input, length, mAType, multiplier, smooth, sTMode);
        }

    }
}
#endregion
